/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */

#ifndef BASLER_CAMERA_DRIVER_DEVICE_H
#define BASLER_CAMERA_DRIVER_DEVICE_H

#include <linux/i2c.h>
#include <linux/regulator/consumer.h>
#ifdef NVIDIA_BUILD
#include <media/camera_common.h>
#endif
#include <linux/version.h>
#include <media/v4l2-ctrls.h>
#include <media/v4l2-device.h>

#include "basler-camera-driver.h"
#include "process-local-ra-storage.h"

#define BASLER_MEMROY_ADDRESS_SIZE_UNKNOWN 0x0
#define BASLER_MEMROY_ADDRESS_SIZE_16BIT 0x2
#define BASLER_MEMROY_ADDRESS_SIZE_32BIT 0x4

struct basler_camera_dev {
	struct i2c_client *i2c_client;
	struct v4l2_subdev* sd;
	struct media_pad pad;
	struct v4l2_mbus_framefmt format;

	/* lock to protect all members below */
	struct mutex lock;

	int power_count;
	struct v4l2_ctrl_handler ctrl_handler;

	bool device_information_cached;
	struct basler_device_information device_information;

	/* Storage for register address and data size for V4L2 ctrl register reads */
	struct process_local_ra_storage_container ra_read_container;

	/* address length in bytes, which are used for later R/W memory */
	atomic_t memory_address_size;

	struct regulator *vdd;

#ifdef NVIDIA_BUILD
	struct camera_common_data *s_data;
#else
	struct v4l2_subdev sd_;
#endif
};


#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
int basler_retrieve_csi_information(struct basler_camera_dev *sensor,
					   struct basler_csi_information *bci);
#else
int basler_retrieve_csi_information(struct basler_camera_dev *sensor,
					   struct basler_csi_information *bci);
#endif

#endif /* BASLER_CAMERA_DRIVER_DEVICE_H */
