// SPDX-License-Identifier: GPL-2.0

#ifndef BASLER_CAMERA_DRIVER_PROCESS_LOCAL_RA_STORAGE_H
#define BASLER_CAMERA_DRIVER_PROCESS_LOCAL_RA_STORAGE_H

#include <linux/types.h>
#include <linux/mutex.h>

#define MAX_RA_STORAGE_ITEMS 8

struct ra_storage_item {
	bool is_used; // false means the entry is unused and any other fields are invalid
	u16 order_number; // 0 .. MAX_RA_STORAGE_ITEMS-1, newer items have bigger numbers
	pid_t item_owner; // the global PID of the process which use the entry

	u32 address;
	u32 data_size;
};

struct process_local_ra_storage_container {
	struct ra_storage_item items[MAX_RA_STORAGE_ITEMS];
	struct mutex lock;
};

void process_local_ra_storage_init(
	struct process_local_ra_storage_container *p_storage);
void process_local_ra_storage_release(
	struct process_local_ra_storage_container *p_storage);

int process_local_ra_storage_store(
	struct process_local_ra_storage_container *p_storage, u32 ra_address,
	u32 ra_data_size);
int process_local_ra_storage_load_and_free(
	struct process_local_ra_storage_container *p_storage, u32 *p_ra_address,
	u32 *p_ra_data_size);

#endif //BASLER_CAMERA_DRIVER_PROCESS_LOCAL_RA_STORAGE_H
